/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.bpel.validation.core;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.netbeans.modules.xml.xam.Component;
import org.netbeans.modules.xml.xam.Model;
import org.netbeans.modules.xml.xam.spi.Validation;
import org.netbeans.modules.xml.xam.spi.Validation.ValidationType;
import org.netbeans.modules.xml.xam.spi.ValidationResult;
import org.netbeans.modules.xml.xam.spi.Validator.ResultType;

import org.netbeans.modules.bpel.model.api.BpelModel;
import org.netbeans.modules.bpel.model.api.CreateInstanceActivity;
import org.netbeans.modules.bpel.model.api.Process;
import org.netbeans.modules.bpel.model.api.support.Initiate;
import org.netbeans.modules.bpel.model.api.support.TBoolean;
import static org.netbeans.modules.soa.ui.util.UI.*;

/**
 * @author Vladimir Yaroslavskiy
 * @version 2008.02.15
 */
public abstract class BpelValidator extends CoreValidator {

  public synchronized ValidationResult validate(Model model, Validation validation, ValidationType type) {
    if ( !(model instanceof BpelModel)) {
      return null;
    }
    final BpelModel bpelModel = (BpelModel) model;
    
    if (bpelModel.getState() == Model.State.NOT_WELL_FORMED) {
      return null;
    }
    init(validation, type);

    Runnable run = new Runnable() {
      public void run() {
        startTime();
        Process process = bpelModel.getProcess();

        if (process != null) {
          process.accept(BpelValidator.this);
        }
        endTime(getDisplayName());
      }
    };
    bpelModel.invoke(run);
//if (getName().contains("custom")) {
//out();
//out();
//out("!!! ERRORS: " + getResultItems().size());
//out();
//out();
//}
    return new ValidationResult(getResultItems(), Collections.singleton(model));
  }

  protected final boolean isCreateInstanceYes(CreateInstanceActivity activity) {
    return activity != null && activity.getCreateInstance() == TBoolean.YES;
  }

  protected final CreateInstanceActivity getCreateInstanceActivity(Component component) {
    if (component instanceof CreateInstanceActivity) {
      return (CreateInstanceActivity) component;
    }
    if (component.getParent() instanceof CreateInstanceActivity) {
      return (CreateInstanceActivity) component.getParent();
    }
    return null;
  }
}
